@@ -136,6 +136,7 @@ The module contains the following public classes:
     - :class:`~fgpyo.sam.SupplementaryAlignment` -- Stores a supplementary alignment record
         produced by BWA and stored in the SA SAM tag.
     - :class:`~fgpyo.sam.SamFileType` -- Enumeration of valid SAM/BAM/CRAM file types.
+    - :class:`~fgpyo.sam.SamOrder` -- Enumeration of possible SAM/BAM/CRAM sort orders.
     - :class:`~fgpyo.sam.CigarOp` -- Enumeration of operators that can appear in a Cigar string.
     - :class:`~fgpyo.sam.CigarElement` -- Class representing an element in a Cigar string.
     - :class:`~fgpyo.sam.CigarParsingException` -- The exception raised specific to parsing a
@@ -169,6 +170,7 @@ from pysam import AlignedSegment
 from pysam import AlignmentFile as SamFile
 from pysam import AlignmentHeader as SamHeader
 
+import fgpyo.io
 from fgpyo.collections import PeekableIterator
 
 SamPath = Union[IO[Any], Path, str]
@@ -259,8 +261,11 @@ def _pysam_open(
     if unmapped and open_for_reading:
         kwargs["check_sq"] = False
 
-    # Open it!
-    return pysam.AlignmentFile(path, **kwargs)
+    # Open it alignment file, suppressing stderr in case index files are older than SAM file
+    with fgpyo.io.suppress_stderr():
+        alignment_file = pysam.AlignmentFile(path, **kwargs)
+    # now restore stderr and return the alignment file
+    return alignment_file
 
 
 def reader(
@@ -374,6 +379,11 @@ class CigarOp(enum.Enum):
         """Returns true if the operator is an indel, false otherwise."""
         return self == CigarOp.I or self == CigarOp.D
 
+    @property
+    def is_clipping(self) -> bool:
+        """Returns true if the operator is a soft/hard clip, false otherwise."""
+        return self == CigarOp.S or self == CigarOp.H
+
 
 @attr.s(frozen=True, slots=True)
 class CigarElement:
@@ -834,6 +844,45 @@ class Template:
             for rec in recs:
                 yield rec
 
+    def write_to(
+        self,
+        writer: SamFile,
+        primary_only: bool = False,
+    ) -> None:
+        """Write the records associated with the template to file.
+
+        Args:
+            writer: An open, writable AlignmentFile.
+            primary_only: If True, only write primary alignments.
+        """
+
+        if primary_only:
+            rec_iter = self.primary_recs()
+        else:
+            rec_iter = self.all_recs()
+
+        for rec in rec_iter:
+            writer.write(rec)
+
+    def set_tag(
+        self,
+        tag: str,
+        value: Union[str, int, float, None],
+    ) -> None:
+        """Add a tag to all records associated with the template.
+
+        Setting a tag to `None` will remove the tag.
+
+        Args:
+            tag: The name of the tag.
+            value: The value of the tag.
+        """
+
+        assert len(tag) == 2, f"Tags must be 2 characters: {tag}."
+
+        for rec in self.all_recs():
+            rec.set_tag(tag, value)
+
 
 class TemplateIterator(Iterator[Template]):
     """
@@ -851,3 +900,14 @@ class TemplateIterator(Iterator[Template]):
         name = self._iter.peek().query_name
         recs = self._iter.takewhile(lambda r: r.query_name == name)
         return Template.build(recs, validate=False)
+
+
+class SamOrder(enum.Enum):
+    """
+    Enumerations of possible sort orders for a SAM file.
+    """
+
+    Unsorted = "unsorted"  #: the SAM / BAM / CRAM is unsorted
+    Coordinate = "coordinate"  #: coordinate sorted
+    QueryName = "queryname"  #: queryname sorted
+    Unknown = "unknown"  # Unknown SAM / BAM / CRAM sort order
