@@ -1,14 +1,20 @@
+import sys
 from typing import Any
 from typing import Dict
+from typing import Iterable
 from typing import List
 from typing import Tuple
 from typing import Type
 from typing import Union
 
-try:  # py>=38
+if sys.version_info >= (3, 8):
     from typing import Literal
-except ImportError:  # py<38
+else:
     from typing_extensions import Literal
+if sys.version_info >= (3, 12):
+    from typing import TypeAlias
+else:
+    from typing_extensions import TypeAlias
 
 import functools
 from enum import Enum
@@ -29,8 +35,8 @@ class ParserNotFoundException(Exception):
 def split_at_given_level(
     field: str,
     split_delim: str = ",",
-    increase_depth_chars: List[str] = ["{", "(", "["],
-    decrease_depth_chars: List[str] = ["}", ")", "]"],
+    increase_depth_chars: Iterable[str] = ("{", "(", "["),
+    decrease_depth_chars: Iterable[str] = ("}", ")", "]"),
 ) -> List[str]:
     """
     Splits a nested field by its outer-most level
@@ -65,7 +71,7 @@ def split_at_given_level(
 
 
 def _get_parser(
-    cls: Type, type_: Type, parsers: Optional[Dict[type, Callable[[str], Any]]] = None
+    cls: Type, type_: TypeAlias, parsers: Optional[Dict[type, Callable[[str], Any]]] = None
 ) -> partial:
     """Attempts to find a parser for a provided type.
 
@@ -114,14 +120,13 @@ def _get_parser(
                     subtypes[0],
                     parsers,
                 )
-                origin_type = types.get_origin_type(type_)
                 return functools.partial(
-                    lambda s: origin_type(
+                    lambda s: list(
                         []
                         if s == ""
                         else [
                             subtype_parser(item)
-                            for item in origin_type(split_at_given_level(s, split_delim=","))
+                            for item in list(split_at_given_level(s, split_delim=","))
                         ]
                     )
                 )
@@ -135,14 +140,13 @@ def _get_parser(
                     subtypes[0],
                     parsers,
                 )
-                origin_type = types.get_origin_type(type_)
                 return functools.partial(
-                    lambda s: origin_type(
+                    lambda s: set(
                         set({})
                         if s == "{}"
                         else [
                             subtype_parser(item)
-                            for item in origin_type(split_at_given_level(s[1:-1], split_delim=","))
+                            for item in set(split_at_given_level(s[1:-1], split_delim=","))
                         ]
                     )
                 )
@@ -155,7 +159,6 @@ def _get_parser(
                     )
                     for subtype in types.get_arg_types(type_)
                 ]
-                origin_type = types.get_origin_type(type_)
 
                 def tuple_parse(tuple_string: str) -> Tuple[Any, ...]:
                     """
@@ -164,7 +167,7 @@ def _get_parser(
                     unpaired '{', or '}' characters
                     """
                     assert tuple_string[0] == "(", "Tuple val improperly formatted"
-                    assert tuple_string[-1] == ")", "Tuple val improprly formatted"
+                    assert tuple_string[-1] == ")", "Tuple val improperly formatted"
                     tuple_string = tuple_string[1:-1]
                     if len(tuple_string) == 0:
                         return ()
@@ -247,14 +250,21 @@ def _get_parser(
     # Set the name that the user expects to see in error messages (we always
     # return a temporary partial object so it's safe to set its __name__).
     # Unions and Literals don't have a __name__, but their str is fine.
-    parser.__name__ = getattr(type_, "__name__", str(type_))
+    setattr(parser, "__name__", getattr(type_, "__name__", str(type_)))
     return parser
 
 
 def attr_from(
     cls: Type, kwargs: Dict[str, str], parsers: Optional[Dict[type, Callable[[str], Any]]] = None
 ) -> Any:
-    """Builds an attr class from key-word arguments"""
+    """Builds an attr class from key-word arguments
+
+    Args:
+        cls: the attr class to be built
+        kwargs: a dictionary of keyword arguments
+        parsers: a dictionary of parser functions to apply to specific types
+
+    """
     return_values: Dict[str, Any] = {}
     for attribute in attr.fields(cls):
         return_value: Any
@@ -267,16 +277,6 @@ def attr_from(
                 return_value = attribute.converter(str_value)
                 set_value = True
 
-            # try setting by casting
-            # Note that while bools *can* be cast from string, all non empty strings evaluate to
-            # True, because python, so we need to check for that explicitly
-            if not set_value and attribute.type is not None and not attribute.type == bool:
-                try:
-                    return_value = attribute.type(str_value)
-                    set_value = True
-                except (ValueError, TypeError):
-                    pass
-
             # try getting a known parser
             if not set_value:
                 try:
@@ -286,17 +286,41 @@ def attr_from(
                 except ParserNotFoundException:
                     pass
 
+            # try setting by casting
+            # Note that while bools *can* be cast from string, all non-empty strings evaluate to
+            # True, because python, so we need to check for that explicitly
+            if not set_value and attribute.type is not None and not attribute.type == bool:
+                try:
+                    return_value = attribute.type(str_value)
+                    set_value = True
+                except (ValueError, TypeError):
+                    pass
+
             # fail otherwise
             assert (
                 set_value
             ), f"Do not know how to convert string to {attribute.type} for value: {str_value}"
         else:  # no value, check for a default
-            assert attribute.default is not None or (
-                types.get_origin_type(attribute.type) is Union
-                and isinstance(None, types.get_arg_types(attribute.type))
+            assert attribute.default is not None or attribute_is_optional(
+                attribute
             ), f"No value given and no default for attribute `{attribute.name}`"
             return_value = attribute.default
+            # when the default is attr.NOTHING, just use None
+            if return_value is attr.NOTHING:
+                return_value = None
 
         return_values[attribute.name] = return_value
 
     return cls(**return_values)
+
+
+def attribute_is_optional(attribute: attr.Attribute) -> bool:
+    """Returns True if the attribute is optional, False otherwise"""
+    return types.get_origin_type(attribute.type) is Union and isinstance(
+        None, types.get_arg_types(attribute.type)
+    )
+
+
+def attribute_has_default(attribute: attr.Attribute) -> bool:
+    """Returns True if the attribute has a default value, False otherwise"""
+    return attribute.default != attr.NOTHING or attribute_is_optional(attribute)
