@@ -1,4 +1,5 @@
 import enum
+import gzip
 from pathlib import Path
 from typing import Any
 from typing import Callable
@@ -10,7 +11,6 @@ from typing import Tuple
 
 import attr
 import pytest
-from py._path.local import LocalPath as TmpDir
 
 from fgpyo.util.metric import Metric
 
@@ -125,9 +125,24 @@ class NamedPerson(Metric["NamedPerson"]):
             return super().format_value(value=value)
 
 
+@attr.s(auto_attribs=True, frozen=True)
+class PersonMaybeAge(Metric["PersonMaybeAge"]):
+    name: str
+    age: Optional[int]
+
+
+@attr.s(auto_attribs=True, frozen=True)
+class PersonDefault(Metric["PersonDefault"]):
+    name: str
+    age: int = 0
+
+
 @pytest.mark.parametrize("metric", DUMMY_METRICS)
-def test_metric_roundtrip(tmpdir: TmpDir, metric: DummyMetric) -> None:
-    path: Path = Path(tmpdir) / "metrics.txt"
+def test_metric_roundtrip(
+    tmp_path: Path,
+    metric: DummyMetric,
+) -> None:
+    path: Path = tmp_path / "metrics.txt"
 
     DummyMetric.write(path, metric)
     metrics: List[DummyMetric] = list(DummyMetric.read(path=path))
@@ -136,8 +151,8 @@ def test_metric_roundtrip(tmpdir: TmpDir, metric: DummyMetric) -> None:
     assert metrics[0] == metric
 
 
-def test_metrics_roundtrip(tmpdir: TmpDir) -> None:
-    path: Path = Path(tmpdir) / "metrics.txt"
+def test_metrics_roundtrip(tmp_path: Path) -> None:
+    path: Path = tmp_path / "metrics.txt"
 
     DummyMetric.write(path, *DUMMY_METRICS)
     metrics: List[DummyMetric] = list(DummyMetric.read(path=path))
@@ -146,6 +161,72 @@ def test_metrics_roundtrip(tmpdir: TmpDir) -> None:
     assert metrics == DUMMY_METRICS
 
 
+def test_metrics_roundtrip_gzip(tmp_path: Path) -> None:
+    path: Path = Path(tmp_path) / "metrics.txt.gz"
+
+    DummyMetric.write(path, *DUMMY_METRICS)
+
+    with gzip.open(path, "r") as handle:
+        handle.read(1)  # Will raise an exception if not a GZIP file.
+
+    metrics: List[DummyMetric] = list(DummyMetric.read(path=path))
+
+    assert len(metrics) == len(DUMMY_METRICS)
+    assert metrics == DUMMY_METRICS
+
+
+def test_metrics_read_extra_columns(tmp_path: Path) -> None:
+    person = Person(name="Max", age=42)
+    path = tmp_path / "metrics.txt"
+    with path.open("w") as writer:
+        header = Person.header()
+        header.append("foo")
+        writer.write("\t".join(header) + "\n")
+        writer.write(f"{person.name}\t{person.age}\tbar\n")
+
+    assert list(Person.read(path=path)) == [person]
+    assert list(Person.read(path=path, ignore_extra_fields=True)) == [person]
+    with pytest.raises(ValueError):
+        list(Person.read(path=path, ignore_extra_fields=False))
+
+
+def test_metrics_read_missing_optional_columns(tmp_path: Path) -> None:
+    person = PersonMaybeAge(name="Max", age=None)
+    path = tmp_path / "metrics.txt"
+
+    # The "age" column is optional, and not in the file, but that's ok
+    with path.open("w") as writer:
+        writer.write("name\nMax\n")
+    assert list(PersonMaybeAge.read(path=path)) == [person]
+
+    # The "age" column is not optional, and not in the file on line 3, and that's not ok
+    with path.open("w") as writer:
+        writer.write("name\tage\nMax\t42\nMax\n")
+    with pytest.raises(ValueError):
+        list(PersonMaybeAge.read(path=path))
+
+
+def test_metric_read_missing_column_with_default(tmp_path: Path) -> None:
+    person = PersonDefault(name="Max")
+    path = tmp_path / "metrics.txt"
+
+    # The "age" column hs a default, and not in the file, but that's ok
+    with path.open("w") as writer:
+        writer.write("name\nMax\n")
+    assert list(PersonDefault.read(path=path)) == [person]
+
+    # All fields specified
+    with path.open("w") as writer:
+        writer.write("name\tage\nMax\t42\n")
+    assert list(PersonDefault.read(path=path)) == [PersonDefault(name="Max", age=42)]
+
+    # Just age specified, but not the required name column!
+    with path.open("w") as writer:
+        writer.write("age\n42\n")
+    with pytest.raises(ValueError):
+        list(PersonDefault.read(path=path))
+
+
 def test_metric_header() -> None:
     assert DummyMetric.header() == [
         "int_value",
@@ -186,7 +267,7 @@ def test_metric_custom_formatter() -> None:
     assert list(person.formatted_values()) == ["john doe", "42"]
 
 
-def test_metric_parse_with_None() -> None:
+def test_metric_parse_with_none() -> None:
     assert Person.parse(fields=["", "40"]) == Person(name=None, age=40)
     assert Person.parse(fields=["Sally", ""]) == Person(name="Sally", age=None)
     assert Person.parse(fields=["", ""]) == Person(name=None, age=None)
@@ -228,7 +309,7 @@ def test_metric_list_format_with_empty_string() -> None:
     )
 
 
-def test_metric_list_parse_with_None() -> None:
+def test_metric_list_parse_with_none() -> None:
     assert ListPerson.parse(fields=[",Sally", "40, 30"]) == ListPerson(
         name=[None, "Sally"], age=[40, 30]
     )
@@ -238,3 +319,27 @@ def test_metric_list_parse_with_None() -> None:
     assert ListPerson.parse(fields=["Max,Sally", ","]) == ListPerson(
         name=["Max", "Sally"], age=[None, None]
     )
+
+
+def test_metrics_fast_concat(tmp_path: Path) -> None:
+    path_input = [
+        tmp_path / "metrics_1.txt",
+        tmp_path / "metrics_2.txt",
+        tmp_path / "metrics_3.txt",
+    ]
+    path_output: Path = tmp_path / "metrics_concat.txt"
+
+    DummyMetric.write(path_input[0], DUMMY_METRICS[0])
+    DummyMetric.write(path_input[1], DUMMY_METRICS[1])
+    DummyMetric.write(path_input[2], DUMMY_METRICS[2])
+
+    Metric.fast_concat(*path_input, output=path_output)
+    metrics: List[DummyMetric] = list(DummyMetric.read(path=path_output))
+
+    assert len(metrics) == len(DUMMY_METRICS)
+    assert metrics[0].header() == DummyMetric.header()
+    assert metrics[1].header() == DummyMetric.header()
+    assert metrics[2].header() == DummyMetric.header()
+    assert metrics[0] == DUMMY_METRICS[0]
+    assert metrics[1] == DUMMY_METRICS[1]
+    assert metrics[2] == DUMMY_METRICS[2]
